///
/// @file        can.h
/// @copyright   Copyright (c) 2015 Toradex AG
/// $Author: andy.kiser $
/// $Rev: 3268 $
/// $Date: 2016-05-18 12:40:03 +0200 (Mi., 18 Mai 2016) $
/// @brief       Library to support CAN on Toradex modules
/// @test        Colibri VFxx, iMX6, Apalis iMX6
/// @target      Colibri VFxx, iMX6, Apalis iMX6
///

/// @page pcfg_can CAN Configuration Options
/// @tableofcontents
///
/// @section scfg_can Compatible CAN Configurations
/// @anchor acfg_can
/// The following table explains the configuration parameters supported by calls of
/// * Can_SetConfigString()
/// * Can_GetConfigString()
/// * Can_SetConfigInt()
/// * Can_GetConfigInt()
///
/// <table class="libParam" >
///   <tr><th>                              Parameter Name </th><th> Value    </th><th> Description</th></tr>
///   <tr><td class="paramname"            >Implementation </td><td> LPWSTR   </td><td> CAN implementation, specific value will depend upon SoC.</td></tr>
///   <tr><td class="paramname"            >Interface      </td><td> LPWSTR   </td><td> Channel used for CAN from the selected implementation, specific value will depend upon SoC.</td></tr>
///   <tr><td class="paramname"            >ioRx<sup><a href="#fn5">5</a></sup>             </td><td> DWORD   </td><td> GPIO used as CAN RX pin in uIo format.</td></tr>
///   <tr><td class="paramname"            >ioTx<sup><a href="#fn5">5</a></sup>             </td><td> DWORD   </td><td> GPIO used as CAN TX pin in uIo format.</td></tr>
///   <tr><td class="paramname"            >BitRateHz      </td><td> DWORD    </td><td> CAN speed in Hz.</td></tr>
///   <tr><td class="paramname"            >Timeout        </td><td> DWORD    </td><td> CAN timeout in ms, INFINITE to wait until operation is complete.</td></tr>
///   <tr><td class="paramname" rowspan="2">FrameFormat                                   </td><td> standard </td><td> 11 bit frame ID.</td></tr>
///   <tr>                                                                                 <td> extended </td><td> 29 bit frame ID.</td></tr>
///   <tr><td class="paramname" rowspan="2">RtrFormat                                   </td><td> data </td><td> RTR data frame.</td></tr>
///   <tr>                                                                              <td> remote </td><td> RTR remote frame.</td></tr>
///   <tr><td class="paramname"            >FilterId
///                                         <sup><a href="#fn1">1</a>,<a href="#fn2">2</a></sup>
///                                                        </td><td> DWORD    </td><td> Frame ID to filter.</td></tr></td></tr>
///   <tr><td class="paramname"            >FilterMask
///                                         <sup><a href="#fn1">1</a>,<a href="#fn2">2</a></sup>
///                                                        </td><td> DWORD    </td><td> Frame ID mask to filter.</td></tr></td></tr>
///   <tr><td class="paramname" rowspan="2">FilterRemote
///                                         <sup><a href="#fn1">1</a></sup><sup>,<a href="#fn2">2</a></sup>
///                                                        </td><td> remote     </td><td> Filter messages for remote frame.</td></tr>
///   <tr>                                                      <td> none    </td><td> Data and remote frame receive.</td></tr>
///   <tr><td class="paramname"            >SpiHandle
///                                         <sup><a href="#fn3">3</a>,<a href="#fn4">4</a></sup>
///                                                        </td><td> DWORD    </td><td> HANDLE to SPI channel used to interface MCP2515 controller, obtained from Spi_Init().</td></tr>
///   <tr><td class="paramname" rowspan="3">BusStatus<sup><a href="#fn4">4</a></sup>
///                                                        </td><td> 0  </td><td> CAN controller is active, without any restrictions</td></tr>
///   <tr>                                                      <td> 1  </td><td> Only Messages and passive error frames may be transmitted.</td></tr>
///   <tr>                                                      <td> 2  </td><td> Messages can neither be transmitted nor received.</td></tr>
/// </table>
/// <sup><ol>
///   <li name="fn1" value=1>Frame filter logic: (FilterID) & (FilterMask) == (received frame ID) & (FilterMask)<br>
///      Any received frame ID that matches with the filter will be returned from CAN read function.
///   <li name="fn2" value="2">This feature is not yet supported. The precise definition of the API is subject to change.
///   <li name="fn3" value=3>Only relevant if "Implementation" is configured to "MCP2515"
///   <li name="fn4" value=4>Read-Only. Get this parameter value <b>after</b> calling Can_Open()
///   <li name="fn5" value=5>Set these parameters <b>before</b> calling Can_Open()
/// </ol></sup>

#ifndef __CAN_H__
#define __CAN_H__

#include <windows.h>
#include "TdxCommon.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum
{
    Active = 0,        ///< CAN controller is active, without any restrictions
    Passive,           ///< Only Messages and passive error frames may be transmitted
    Off                ///< Messages can neither be transmitted nor received
} tCanBusStatus;

//******************************************************************************
// Data types used for CAN communication
//******************************************************************************
///@anchor IDE
#define CanMsgFlags_IDE 0x00000001  ///< Identifier extension bit (IDE), used for tCanMsg.canMsgFlags\n
                                    ///< 0 = id is in standard format (11 bits)\n
                                    ///< 1 = id is in extended format (29 bits)
///@anchor RTR
#define CanMsgFlags_RTR 0x00000002  ///< Remote transmission request (RTR), used for tCanMsg.canMsgFlags\n
                                    ///< 0 = message is a regular data frame\n
                                    ///< 1 = message is a remote data request frame

/// Representation of a CAN message.
typedef struct
{
    DWORD           id;             ///< ID for CAN Bus Message Buffer (11 or 29bits used)
    DWORD           canMsgFlags;    ///< message specific flags.Currently only 2 bits are used:\n * @ref IDE "CanMsgFlags_IDE"\n * @ref RTR "CanMsgFlags_RTR" (read only)
    DWORD           dataLen;        ///< Message Data Length (0...8). This matches the DLC field of a CAN message.
    BYTE            data[8];        ///< Message Data (0...8 bytes used). This matches the data field of a CAN message.
} tCanMsg, 
  *pCanMsg;                         ///< Pointer to a tCanMsg

//******************************************************************************
/// Initialize the CAN channel.\n
/// This function call does not modify any hardware registers. Therefore the
/// returned handle can be used to make additional configuration settings, which
/// will become active when calling Can_Open().
/// @param[in]    portName        CAN channel name. E.g. "CAN1", "SPI2"
/// @return                       Handle to the CAN port. NULL if the function failed.
HANDLE Can_Init(const TCHAR *portName);

//******************************************************************************
/// De-initialize the HANDLE. The port must be closed before.
/// @param[in]    hPort           HANDLE received from Can_Init()
/// @retval       TRUE            Success
/// @retval       FALSE           Failure
BOOL Can_Deinit(HANDLE hPort);

//******************************************************************************
/// Apply configuration to the hardware.
/// @param[in]    hPort            HANDLE received from Can_Init()
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Can_Open(HANDLE hPort);

//******************************************************************************
/// Close CAN
/// @param[in]    hPort            HANDLE received from Can_Init()
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Can_Close(HANDLE hPort);

//******************************************************************************
/// Read one CAN message.\n
/// Can messages are buffered in a receive buffer. This function reads one message from the buffer.
/// @param[in]    hPort            HANDLE received from Can_Init()
/// @param[out]   msg              Buffer to store the received message.
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Can_Read(HANDLE hPort, pCanMsg msg);

//******************************************************************************
/// Write one CAN Message.\n
/// @param[in]    hPort            HANDLE received from Can_Init()
/// @param[in]    msg              Pointer to the message to be sent
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Can_Write(HANDLE hPort, pCanMsg msg);

//******************************************************************************
/// Retrieves a DWORD configuration parameter.
/// @param[in]    hPort            HANDLE received from Can_Init()
/// @param[in]    paramName        Parameter value to retrieve\n
///                                Refer to the @ref scfg_can for a list of supported parameters.
/// @param[out]   pValue           Data obtained
/// @return                        Number of bytes read. 0 if paramName not found.
DWORD Can_GetConfigInt(HANDLE hPort, const TCHAR *paramName, DWORD *pValue);

//******************************************************************************
/// Set a DWORD configuration parameter.
/// @param[in]    hPort            HANDLE received from Can_Init()
/// @param[in]    paramName        Parameter value to configure.\n
///                                Refer to the @ref scfg_can for a list of supported parameters.
/// @param[in]    value            Data to be set
/// @param[in]    storageType      StoreToRegistry = parameter will also be stored in the registry\n
///                                StoreVolatile   = parameter will only be stored in local variable.
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Can_SetConfigInt(HANDLE hPort, const TCHAR *paramName, DWORD value, ParamStorageType storageType);

//******************************************************************************
/// Retrieves the current string configuration parameter.
/// @param[in]    hPort            HANDLE received from Can_Init()
/// @param[in]    paramName        Pointer to a string containing the name of the string field to get.
///                                Refer to @ref scfg_can for details.
/// @param[out]   pValue           Pointer to a buffer that receives the value's string data
/// @param[in]    maxBytes         Maximum size of string in bytes
/// @return                        Number of bytes read. 0 indicates an error.
DWORD Can_GetConfigString(HANDLE hPort, const TCHAR *paramName, TCHAR *pValue, DWORD maxBytes);

//******************************************************************************
/// Set string configuration parameters.\n
/// @param[in]    hPort            HANDLE received from Can_Init()
/// @param[in]    paramName        Pointer to a string containing the name of the string field to set.
///                                Refer to @ref scfg_can for details.
/// @param[in]    pValue           String value to be set
/// @param[in]    storageType      StoreToRegistry = parameter will also be stored in the registry\n
///                                StoreVolatile   = parameter will only be stored in local variable.
/// @retval       TRUE             Success
/// @retval       FALSE            Failure
BOOL Can_SetConfigString(HANDLE hPort, const TCHAR *paramName, const TCHAR *pValue, ParamStorageType storageType);

//******************************************************************************
/// Get the revision of this CAN library
/// @param[out]   ver              library version, consisting of major, minor and build version
void Can_GetVersion(tVersion *ver);

#ifdef __cplusplus
}
#endif

#endif