///
/// @file        pwm_imx6.h
/// @copyright   Copyright (c) 2014 Toradex AG
/// $Author: raja.sivasankaran $
/// $Rev: 3075 $
/// $Date: 2016-01-25 12:45:00 +0530 (Mon, 25 Jan 2016) $
/// @brief       Library to support Pulse Width Modulation (PWM) on iMX6.
/// @test
/// @target      Colibri iMX6S, iMX6DL, Apalis iMX6Q, iMX6D
///
/// @anchor imx6pwm_pins
/// <h2 class ="groupheader">PWM Pin Availability</h2>
/// The tables below list the module pins, on which PWM functionality is available.\n
/// <h3>Colibri iMX6</h3>
/// The default PWM channels as defined on the Colibri Evaluation board are listed
/// in the column "PWM".
///
/// |   SODIMM   |  GPIO  |  AltFn  |    PWM     | PWM Channel |
/// | :--------: | :----: | :-----: | :--------: | :----------:|
/// |     28     |   09   |     4   | PWM1/PWM_B |        1    |
/// |     67     |   01   |     4   | PWM2/PWM_D |        2    |
/// |     59     |   41   |     2   | PWM3/PWM_A |        3    |
/// |     30     |   42   |     2   | PWM4/PWM_C |        4    |
/// |     53     |  125   |     2   |            |        1    |
/// |     51     |  126   |     2   |            |        2    |
/// |     49     |   17   |     2   |            |        3    |
/// |    190     |   18   |     2   |            |        4    |
/// |     62     |   21   |     3   |            |        1    |
/// |     48     |   19   |     3   |            |        2    |
///
/// <h3>Apalis iMX6</h3>
/// |    MXM3    |  GPIO  |  AltFn  |    PWM     | PWM Channel |
/// | :--------: | :----: | :-----: | :--------: | :----------:|
/// |      2     |   09   |     4   |    PWM1    |        1    |
/// |      4     |   01   |     4   |    PWM2    |        2    |
/// |      6     |   41   |     2   |    PWM3    |        3    |
/// |      8     |   42   |     2   |    PWM4    |        4    |
/// |    239     |  125   |     2   |    PWM4    |        4    |
/// _NOTE:_ The PWM4(MXM3 8) and BK1_PWM(MXM3 239) are shared PWM output pins.
///
/// @page    pcfg_pwm      PWM Configuration Options
/// @section scfg_pwm_imx6  iMX6 PWM Configurations
/// @anchor  acfg_pwm_imx6
/// The following table explains the configuration parameters supported by calls of
/// * Imx6Pwm_SetConfigInt()
/// * Imx6Pwm_GetConfigInt()
/// * Imx6Pwm_SetConfigString()
/// * Imx6Pwm_GetConfigString()
/// <table class="libParam" >
/// <tr><th>valueName</th>  <th>Type</th><th>Description</th></tr>
/// <tr><td>io</td>         <td>DWORD</td><td>GPIO or Pin number.<br>
///                         The higher 16bit contain the type of the value:
///                         <ul>
///                         <li>0x0010 = GPIO</li>
///                         <li>0x0020 = Colibri SODIMM pin</li>
///                         <li>0x0030 = Apalis MXM3 pin</li>
///                         </ul>
///                         The lower 16bit contain the Gpio number or pin number.<p>
///                         Examples:
///                         <ul>
///                         <li>0x0010003b defines GPIO pin 59</li>
///                         <li>0x00200016 defines Colibri SODIMM pin 22</li>
///                         <li>0x00300002 defines Apalis MXM3 pin 2</li>
///                         </ul>
///                         The PWM hardware pins are same for the Colibri iMX6S/iMX6DL.<p>
///                         The PWM hardware pins are same for the Apalis iMX6Q/iMX6D.<p>
///                         You can define *io* as uIo structure (include Gpio.h) and pass *io.GenericDefinition*.\n
///                         The @ref imx6pwm_pins "Imx6 PWM pins" shows a list of GPIOs with PWM functionality.</td></tr>
/// <tr><td>baseFreq</td>   <td>DWORD</td><td> Base clock frequency for the PWM generator in Hz.<p>
///                         This frequency defines the time resolution of the PWM signal (resolution = 1/baseFreq)<br>
///                         Number of discrete frequencies can be configured in the iMX6. Allowed frequencies are:
///                         <ul>
///                         <li>66.0 MHz/n, where n is 1 to 4096.</li>
///                         </ul>
///                         If baseFreq is not one of these frequencies, the PwmLib will select the next higher frequency.\n
///                         A PWM period can be configured to be 1 to 65536 cycles of the baseFreq. Therefore the lowest achievable PWM frequency is baseFreq/65536.</td></tr>
/// </table>
///
/// @test        Colibri iMX6S, iMX6DL, Apalais iMX6Q, iMX6D
/// @target      iMX6
///

#ifndef __PWM_IMX6_H__
#define __PWM_IMX6_H__

#include <windows.h>
#include "tdxcommon.h"

#ifdef __cplusplus
extern "C" {
#endif


//******************************************************************************
/// Initialize HANDLE for the PWM channel.
/// This function call does not modify any hardware registers. Therefore the
/// returned handle can be used to make additional configuration settings, which
/// will become active when calling Imx6Pwm_Open()
/// @param[in]  portName    PWM channel name. Can be one of these: <ul><li>L"PWM1"</li><li>L"PWM2"</li><li>L"PWM3"</li><li>L"PWM4"</li></ul>
/// @return     Handle to the PWM channel. NULL if the function failed.
HANDLE Imx6Pwm_Init(const TCHAR *portName);

//******************************************************************************
/// Finally close the HANDLE. The port must be closed before.
/// @param[in]    hPort       HANDLE to PWM channel
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL Imx6Pwm_Deinit(HANDLE hPort);

//******************************************************************************
/// Apply configuration to the hardware registers and IOs
/// @param[in]    hPort       HANDLE to PWM channel
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL Imx6Pwm_Open(HANDLE hPort);

//******************************************************************************
/// Close the PWM port
/// @param[in]    hPort       HANDLE to PWM channel
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL Imx6Pwm_Close(HANDLE hPort);

//******************************************************************************
/// Retrieves a string configuration parameter\n
/// There is no String parameter for the Imx6 PWM library, so this function is not used.
/// @param[in]    hPort       HANDLE to PWM channel
/// @param[in]    paramName   Parameter value to retrieve\n
///                           Refer to the @ref scfg_pwm_imx6 for a list of supported parameters.
/// @param[out]   pValue      String data obtained
/// @param[in]    maxBytes    Maximum size of string in Bytes
/// @return                   Number of bytes read. 0 if paramName not found.
DWORD Imx6Pwm_GetConfigString(HANDLE hPort, const TCHAR *paramName, TCHAR *pValue, DWORD maxBytes);

//******************************************************************************
/// Set a string configuration parameter
/// There is no String parameter for the Imx6 PWM library, so this function is not used.
/// @param[in]    hPort       HANDLE to PWM channel
/// @param[in]    paramName   Parameter value to configure\n
///                           Refer to the @ref scfg_pwm_imx6 for a list of supported parameters.
/// @param[in]    value       String value to be set
/// @param[in]    storageType StoreToRegistry = parameter will also be stored in the registry\n
///                           StoreVolatile   = parameter will only be stored in local variable.
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL Imx6Pwm_SetConfigString(HANDLE hPort, const TCHAR *paramName, const TCHAR *value, ParamStorageType storageType);

//******************************************************************************
/// Retrieves a DWORD configuration parameter\n
/// @param[in]    hPort       HANDLE to PWM channel
/// @param[in]    paramName   Parameter value to retrieve\n
///                           Refer to the @ref scfg_pwm_imx6 for a list of supported parameters.
/// @param[out]   pValue      String data obtained
/// @return                   Number of bytes read. 0 if paramName not found.
DWORD Imx6Pwm_GetConfigInt( HANDLE hPort, const TCHAR *paramName, DWORD *pValue);

//******************************************************************************
/// Set a DWORD configuration parameter.\n
/// @param[in]    hPort       HANDLE to PWM channel
/// @param[in]    paramName   Parameter value to configure\n
///                           Refer to the @ref scfg_pwm_imx6 for a list of supported parameters.
/// @param[in]    value       Data to be set
/// @param[in]    storageType StoreToRegistry = parameter will also be stored in the registry\n
///                           StoreVolatile   = parameter will only be stored in local variable.
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL Imx6Pwm_SetConfigInt(HANDLE hPort, const TCHAR *paramName, DWORD value, ParamStorageType storageType);

//******************************************************************************
/// Assigns frequency and duty cycle in PWM Controller. There are only discrete
/// values possible for frequency and duty cycle, so the requested value may not
/// be reached accurately.\n
/// @param[in]    hPort       HANDLE to PWM channel
/// @param[in]    freq        PWM frequency in Hz\n
/// @param[in]    duty        16 bit duty cycle (0 = 0% ... 65536 = 100%)
/// @retval       TRUE        Success
/// @retval       FALSE       Failure
BOOL Imx6Pwm_SetPwm(HANDLE hPort, DWORD freq, DWORD duty);

//******************************************************************************
/// Get the revision of this pwm_imx6 library
/// @param[out]     ver     library version, consist of major, minor and build version
void Imx6Pwm_GetVersion(tVersion *ver);

#ifdef __cplusplus
}
#endif

#endif //__PWM_VYB_H__